// C Source File
// Created 7/14/01; 3:22:00 PM
/*
sstart by Greg Dietsche gdietsche@email.com
based on start and ttstart found at tict.ticalc.org

Features:
Breaks the assembly language program size imposed by different ams versions.
Starts NOSTUB ASM programs.
Starts PPG (Compressed) programs.
Lots of Error Detection Code. - You can't accidently start a doorsos program...

Future Features:
I have some ideas to support doors os asm and huffman compressed files...
but it would take quite a bit of work...

					***Change Log***
07/15/2001 - Initial Version 1.0 1785 bytes
*/

#define NO_EXIT_SUPPORT				// for size and ghost space...
#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#include <tigcclib.h>         // Include All Header Files
#include "ttunpack.h"					// ppg decompression routine
#include "sstart.h"						// header file for sstart.c


short _ti89;                  // Produce .89Z File
short _ti92plus;              // Produce .9XZ File

// Main Function
void _main(void)
{
	short isAsm = FALSE;
	unsigned short length;
	char tmpstr[75], *fname;
	unsigned char *src, *dest;
	volatile HANDLE h;
	ESI argptr;
	SYM_ENTRY *SymPtr;
	
	#ifdef LeakWatch
		unsigned long freemem = HeapAvail();
	#endif
   
	InitArgPtr(argptr);
	
	if (GetArgType(argptr) != STR_TAG)
		fatal(TXT_HELP);
    
//This is intentional... the estack pointer will be changed to skip the 
//first arg so the called prog does not get that argument
//This prevents errors with some programs
	fname = GetStrnArg(top_estack);

//ignore anything after the last (
	if((src = strchr(fname,'(')))	//note to self: using src out of context...
  	*src = 0;
       
//convert to TIOS sytle file name
	tmpstr[0] = 0;
	strcpy(tmpstr+1, fname);
	
//Find the program/ppg in the vat... exit on error
	if(!(SymPtr = DerefSym(SymFind(tmpstr+strlen(fname)+1))))
	{
		sprintf(tmpstr, TXT_FILE_NOT_FOUND, fname);
		fatal(tmpstr);
	}

//ASM or ppg program?
	if(*(unsigned char*)(src = HToESI(h = SymPtr->handle)) == ASM_TAG)
		isAsm = TRUE;
	else
		if(* src    != OTH_TAG ||
       *(src-4) != 'p'     ||
       *(src-3) != 'p'     ||
       *(src-2) != 'g')
			fatal(TXT_FILE_TYPE);

//Get a pointer to the input data
	src = HLock(h);
	
	if(isAsm)
	{	
	//Get the length of the source file
		length = *(short *)src;
				
		if(SymPtr->flags.bits.archived || SymPtr->flags.bits.locked)
		{
		
			//HeapCompress();	//slows down the program some, but can give the called program more ram to use
			
			if(!(dest = malloc(length+2)))
				EXIT_LOW_RAM();

			memcpy(dest+2, src+2, length);
		}
		else
			dest = src;	// can run the program from its current location
	}
	else
	{
		ST_helpMsg(TXT_DECOMPRESSING);
		
		src+=2;
		length = ttunpack_size(src);

		//HeapCompress();	//slows down the program some, but can give the called program more ram to use
		
		if(!(dest = malloc(length)))
			EXIT_LOW_RAM();
			
		if(!ttunpack_valid(src) || ttunpack_decompress(src, dest))
			EXIT_BAD_PPG();
		
		length-=2;
		
		ST_eraseHelp();
	}

//don't need the source to be locked anymore
	if(dest != src)
		HeapUnlock(h);
		
//run the program
	dest+=0x40002;
	enter_ghost_space();
	EX_patch(dest, dest + length - 1);
	ASM_NOP_CALL(dest);
		
//Finish up...
	dest-=0x40002;
	
	if(dest != src)	//if != then ram was allocated so free it
		free(dest);
	else
		HeapUnlock(h);
		
	#ifdef LeakWatch
		if((freemem=HeapAvail()-freemem) != 0)
		{
			push_longint(freemem);
			HomePushEStack();

			sprintf(tmpstr, TXT_LEAK_WATCH, freemem);	
			ST_helpMsg(tmpstr);
		}
	#endif
}
